@@ -1,3 +1,5 @@
+import sys
+
 import pytest
 from pip._internal.models.format_control import FormatControl
 
@@ -34,6 +36,7 @@ def writer(tmpdir_cwd):
             emit_index_url=True,
             emit_trusted_host=True,
             annotate=True,
+            annotation_style="split",
             generate_hashes=False,
             default_index_url=None,
             index_urls=[],
@@ -42,6 +45,8 @@ def writer(tmpdir_cwd):
             allow_unsafe=False,
             find_links=[],
             emit_find_links=True,
+            strip_extras=False,
+            emit_options=True,
         )
         yield writer
 
@@ -53,21 +58,21 @@ def test_format_requirement_annotation_editable(from_editable, writer):
 
     assert writer._format_requirement(
         ireq
-    ) == "-e git+git://fake.org/x/y.git#egg=y\n" + comment("    # via xyz")
+    ) == "-e git+git://fake.org/x/y.git#egg=y\n    " + comment("# via xyz")
 
 
 def test_format_requirement_annotation(from_line, writer):
     ireq = from_line("test==1.2")
     ireq.comes_from = "xyz"
 
-    assert writer._format_requirement(ireq) == "test==1.2\n" + comment("    # via xyz")
+    assert writer._format_requirement(ireq) == "test==1.2\n    " + comment("# via xyz")
 
 
 def test_format_requirement_annotation_lower_case(from_line, writer):
     ireq = from_line("Test==1.2")
     ireq.comes_from = "xyz"
 
-    assert writer._format_requirement(ireq) == "test==1.2\n" + comment("    # via xyz")
+    assert writer._format_requirement(ireq) == "test==1.2\n    " + comment("# via xyz")
 
 
 def test_format_requirement_for_primary(from_line, writer):
@@ -75,7 +80,7 @@ def test_format_requirement_for_primary(from_line, writer):
     ireq = from_line("test==1.2")
     ireq.comes_from = "xyz"
 
-    assert writer._format_requirement(ireq) == "test==1.2\n" + comment("    # via xyz")
+    assert writer._format_requirement(ireq) == "test==1.2\n    " + comment("# via xyz")
 
 
 def test_format_requirement_for_primary_lower_case(from_line, writer):
@@ -83,7 +88,7 @@ def test_format_requirement_for_primary_lower_case(from_line, writer):
     ireq = from_line("Test==1.2")
     ireq.comes_from = "xyz"
 
-    assert writer._format_requirement(ireq) == "test==1.2\n" + comment("    # via xyz")
+    assert writer._format_requirement(ireq) == "test==1.2\n    " + comment("# via xyz")
 
 
 def test_format_requirement_environment_marker(from_line, writer):
@@ -148,7 +153,7 @@ def test_iter_lines__hash_missing(capsys, writer, from_line):
 
     expected_lines = (
         MESSAGE_UNHASHED_PACKAGE,
-        "file:///example/#egg=example",
+        "example @ file:///example/",
         "test==1.2 \\\n    --hash=FAKEHASH",
     )
     assert tuple(lines) == expected_lines
@@ -173,8 +178,8 @@ def test_iter_lines__no_warn_if_only_unhashable_packages(writer, from_line):
     lines = writer._iter_lines(ireqs, hashes=hashes)
 
     expected_lines = (
-        "file:///unhashable-pkg1/#egg=unhashable-pkg1",
-        "file:///unhashable-pkg2/#egg=unhashable-pkg2",
+        "unhashable-pkg1 @ file:///unhashable-pkg1/",
+        "unhashable-pkg2 @ file:///unhashable-pkg2/",
     )
     assert tuple(lines) == expected_lines
 
@@ -184,7 +189,8 @@ def test_write_header(writer):
         comment,
         [
             "#",
-            "# This file is autogenerated by pip-compile",
+            "# This file is autogenerated by pip-compile with python "
+            f"{sys.version_info.major}.{sys.version_info.minor}",
             "# To update, run:",
             "#",
             "#    pip-compile --output-file={} src_file src_file2".format(
@@ -202,7 +208,8 @@ def test_write_header_custom_compile_command(writer, monkeypatch):
         comment,
         [
             "#",
-            "# This file is autogenerated by pip-compile",
+            "# This file is autogenerated by pip-compile with python "
+            f"{sys.version_info.major}.{sys.version_info.minor}",
             "# To update, run:",
             "#",
             "#    ./pipcompilewrapper",
@@ -222,6 +229,37 @@ def test_write_header_no_emit_header(writer):
         next(writer.write_header())
 
 
+@pytest.mark.parametrize(
+    ("emit_options", "expected_flags"),
+    (
+        pytest.param(
+            True,
+            (
+                "--index-url https://index-server",
+                "--find-links links",
+                "--trusted-host index-server",
+                "--no-binary flask",
+                "--only-binary django",
+                "",
+            ),
+            id="on",
+        ),
+        pytest.param(False, (), id="off"),
+    ),
+)
+def test_write_flags_emit_options(writer, emit_options, expected_flags):
+    """
+    There should be options if emit_options is True
+    """
+    writer.emit_options = emit_options
+    writer.index_urls = ["https://index-server"]
+    writer.find_links = ["links"]
+    writer.trusted_hosts = ["index-server"]
+    writer.format_control = FormatControl(no_binary=["flask"], only_binary=["django"])
+
+    assert tuple(writer.write_flags()) == expected_flags
+
+
 def test_write_format_controls(writer):
     """
     Tests --no-binary/--only-binary options.
@@ -339,3 +377,24 @@ def test_write_find_links(writer, find_links, expected_lines):
     """
     writer.find_links = find_links
     assert list(writer.write_find_links()) == expected_lines
+
+
+def test_write_order(writer, from_line):
+    """
+    Order of packages should match that of `pip freeze`.
+    """
+    writer.emit_header = False
+
+    packages = [
+        from_line("package_a==0.1"),
+        from_line("Package-b==2.3.4"),
+        from_line("Package==5.6"),
+        from_line("package2==7.8.9"),
+    ]
+    expected_lines = [
+        "package==5.6",
+        "package_a==0.1",
+        "package-b==2.3.4",
+        "package2==7.8.9",
+    ]
+    assert list(writer._iter_lines(packages)) == expected_lines
